window.MultipleChoiceTestData = (function() {
    // Schon erzeugte Zufallszahlen
    var rands = [];
    // Zahl nach Vorgabe erzeugen
    function getNumber(
        boolRandom,     // Zufällige Zahl (true) oder Null (false)
        negative,       // Soll die Zahl negativ sein?
        showSign        // Unnötige Vorzeichen explizit anzeigen?
    ) {
        if (!boolRandom)
            return 0;
        return buildNumber(getRandomDigits(), negative, showSign);
    }

    // Aus Betrag, Vorzeichen und Anzeigeart für Vorzeichen eine Zahl zusammenbauen
    function buildNumber(
        abs,            // Betrag der Zahl
        negative,       // Soll die Zahl negativ sein?
        showSign        // Unnötige Vorzeichen explizit anzeigen?
    ) {
        return (negative ? '-' : (showSign ? '+' : '')) + abs;
    }

    // Zufällige Ziffernfolge erstellen
    function getRandomDigits() {
        var r;
        do {
            r = Math.floor(Math.random() * Math.pow(10, Math.floor(Math.random() * 3) + 1)) + 1;
        } while (rands.indexOf(r) != -1);
        rands.push(r);
        return r;
    }

    // Zufällig eine Zahl erzeugen
    function getRandomNumber() {
        switch(Math.floor(Math.random() * 9)) {
            case 0:
            case 1:
            case 5:
            case 6:
                return getNumber(true, true, true);
            case 2:
                return '0';
            case 3:
            case 4:
                return getNumber(true, false, true);
            case 7:
            case 8:
                return getNumber(true, false, false);
        }
    }

    // Vorzeichen ermitteln
    function sgn(
        number      // Fragliche Zahl
    ) {
        return (number == 0) ? 0 : ((number > 0) ? +1 : -1);
    }

    // Zufällig eine ggf. ähnliche Zahl erzeugen
    function getSimilarNumber(
        number      // Zahl, zu der eine ähnliche Zahl erzeugt werden soll
    ) {
        // Bei Null: Zahl ungleich Null erzeugen und zurückgeben
        if (number == 0) {
            do
                number = getRandomNumber();
            while (number == 0);
            return number;
        }

        var sign = sgn(number);
        var abs = Math.abs(number);

        if ((sign > 0) && (Math.random() < 0.3))
            return buildNumber(abs, sign, number.indexOf('+') == -1);

        switch(Math.floor(Math.random() * 5)) {
            case 0:
            case 1:
                sign *= -1;
                break;
            case 2:
            case 3:
                abs = getRandomDigits();
                break;
            case 4:
                return getRandomNumber();
        }

        return buildNumber(abs, sign == -1, number.indexOf('+') == -1);
    }

    var data = {
        // Optional: Wie viele Versuche hat der/die Lernende (Standardwert: Unendlich)
        "maxTries": Infinity,

        // Optional: Soll statt der Anzahl an nötigen Versuchen bewertet werden, wie gut der/die Lernende abschneidet?
        "reportScore": false,

        // Optional: Sollen die Fragen nicht durchgemischt werden?
        "keepOrder": false,

        // Optional: Wie soll bewertet werden?
        // "questions"  → Standardwert, Frage kann entweder richtig sein (1 von 1 Punkt) oder falsch (0 von 1 Punkt)
        // "average"    → Durchschnittlicher Prozentsatz an richtig ausgewählten bzw. richtig nicht ausgewählten Antwortmöglichkeiten pro auswählbaren Antwortmöglichkeiten bei jeder Frage
        // "choices"    → Anteil der richtig ausgewählten Antwortmöglichkeiten pro auswählbaren Antwortmöglichkeiten
        "countMode": "questions",

        // Wie viele Fragen sollen für einen Test gezogen werden?
        "questionsPerTest": -1,

        // Welcher Anteil muss mindestens richtig sein, um zu bestehen?
        "threshold": 1,

        // Optional: Begrüßungstext
        "htmlIntro": "<p>Beantworte alle Fragen richtig, um dieses Lernpaket erfolgreich abzuschließen.</p>",

        // Verfügbare statische Fragen
        "questions": [],

        // Fragengeneratoren
        "generators": []
    };

    // Frage nach Betrag
    data.generators.push(function(arrFragen) {
        var number = getRandomNumber();
        frage = {
            "title": "Betrag",
            "html": "Was ist der <strong>Betrag</strong> von <code>" + number + "</code>?",
            "singlechoice": true,
            "choices": [
                { "html": "Der Betrag ist ", "unit": ".", "type": "number", "step": 1, "accept": Math.abs(+number), "correct": true, "size": 2 }
            ]
        };
        arrFragen.push(frage);
    });
    ++data.questionsPerTest;

    // Frage nach Zahl
    data.generators.push(function(arrFragen) {
        var number = getRandomNumber();
        frage = {
            "title": "Zahl",
            "html": "Was ist bei <code>" + number + "</code> die <strong>Zahl</strong>?",
            "singlechoice": true,
            "choices": [
                { "html": "Die Zahl ist ", "unit": ".", "type": "number", "step": 1, "accept": +number, "correct": true, "size": 2 }
            ]
        };
        arrFragen.push(frage);
    });
    ++data.questionsPerTest;

    // Frage nach Gegenzahl
    data.generators.push(function(arrFragen) {
        var number = getRandomNumber();
        frage = {
            "title": "Gegenzahl",
            "html": "Was ist die <strong>Gegenzahl</strong> zu <code>" + number + "</code>?",
            "singlechoice": true,
            "choices": [
                { "html": "Die Gegenzahl ist ", "unit": ".", "type": "number", "step": 1, "accept": -number, "correct": true, "size": 2 }
            ]
        };
        arrFragen.push(frage);
    });
    ++data.questionsPerTest;

    // Frage nach Vorzeichen
    data.generators.push(function(arrFragen) {
        var number = getRandomNumber();
        frage = {
            "title": "Vorzeichen",
            "html": "Was ist das <strong>Vorzeichen</strong> von <code>" + number + "</code>?",
            "singlechoice": true,
            "choices": [
                { "html": "Vorzeichen: ", "type": "text", "correct": true, "casesensitive": false }
            ]
        };
        if (number == 0)
            frage.choices[0].accept = ['keins', 'kein', '', 'nichts', 'kein Vorzeichen', 'nicht vorhanden'];
        else if (+number > 0)
            frage.choices[0].accept = ['Plus', '+'];
        else
            frage.choices[0].accept = ['Minus', '-'];
        arrFragen.push(frage);
    });
    ++data.questionsPerTest;

    // Frage nach Gemeinsamkeiten
    data.generators.push(function(arrFragen) {
        var n1 = getRandomNumber();
        var n2 = getSimilarNumber(n1);
        frage = {
            "title": "Gemeinsamkeiten",
            "html": "Was sind <strong>Gemeinsamkeiten</strong> von <code>" + n1 + "</code> und <code>" + n2 + "</code>?",
            "choices": [
                { "html": "Gleiches Vorzeichen", "correct": (sgn(n1) == sgn(n2)) },
                { "html": "Gleiche Zahl", "correct": (+n1 == +n2) },
                { "html": "Gleiche Gegenzahl", "correct": (+n1 == +n2) },
                { "html": "Gleicher Betrag", "correct": (Math.abs(n1) == Math.abs(n2)) },
                { "html": "Es gibt keine Gemeinsamkeiten", "correct": !((sgn(n1) == sgn(n2)) || (Math.abs(n1) == Math.abs(n2))) }
            ]
        };
        arrFragen.push(frage);
    });
    ++data.questionsPerTest;

    // Frage nach Unterschieden
    data.generators.push(function(arrFragen) {
        var n1 = getRandomNumber();
        var n2 = getSimilarNumber(n1);
        frage = {
            "title": "Unterschiede",
            "html": "Was sind <strong>Unterschiede</strong> von <code>" + n1 + "</code> und <code>" + n2 + "</code>?",
            "choices": [
                { "html": "Unterschiedliche Vorzeichen", "correct": !(sgn(n1) == sgn(n2)) },
                { "html": "Unterschiedliche Zahlen", "correct": !(+n1 == +n2) },
                { "html": "Unterschiedliche Gegenzahlen", "correct": !(+n1 == +n2) },
                { "html": "Unterschiedlicher Betrag", "correct": !(Math.abs(n1) == Math.abs(n2)) },
                { "html": "Es gibt keine Unterschiede", "correct": (+n1 == +n2) }
            ]
        };
        arrFragen.push(frage);
    });
    ++data.questionsPerTest;

    // Frage nach Zahlenart
    data.generators.push(function(arrFragen) {
        var number = getRandomNumber();
        frage = {
            "title": "Zahlenart",
            "html": "Kreuze alle Beschreibungen an, die auf <code>" + number + "</code> zutreffen:",
            "choices": [
                { "html": "<code>" + number + "</code> ist eine <em>natürliche Zahl</em>", "correct": (sgn(number) > 0) },
                { "html": "<code>" + number + "</code> ist eine <em>positive Zahl</em>", "correct": (sgn(number) > 0) },
                { "html": "<code>" + number + "</code> ist eine <em>negative Zahl</em>", "correct": (sgn(number) < 0) },
                { "html": "<code>" + number + "</code> ist eine <em>ganze Zahl</em>", "correct": true }
            ]
        };
        arrFragen.push(frage);
    });
    ++data.questionsPerTest;

    return data;
})();
